////////////////////////////////////////////////////////////////////////////////
//
//  BitmapViewerDoc.cpp
//
//  This source file is a supplement to the article "Windows i Bitmape".
//  The article was published in the #23 and #25 issues of the "PC" computer
//  monthly, published in Belgrade, Yugoslavia by "PC Press"
//
//  You can freely use, modify and redistribute this file, provided that
//  this header is kept unchanged
//
//                                                                 Ivica Nikolic
//                                                      ivica@galeb.etf.bg.ac.yu
//
////////////////////////////////////////////////////////////////////////////////

// BitmapViewerDoc.cpp : implementation of the CBitmapViewerDoc class
//

#include "stdafx.h"
#include "BitmapViewer.h"

#include "BitmapViewerDoc.h"

#include "BitmapInfoDlg.h"

#include "Dib\DibUt.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CBitmapViewerDoc

IMPLEMENT_DYNCREATE(CBitmapViewerDoc, CDocument)

BEGIN_MESSAGE_MAP(CBitmapViewerDoc, CDocument)
	//{{AFX_MSG_MAP(CBitmapViewerDoc)
	ON_COMMAND(ID_CONVERTTO_1BPP, OnConvertto1bpp)
	ON_UPDATE_COMMAND_UI(ID_CONVERTTO_1BPP, OnUpdateConvertto1bpp)
	ON_COMMAND(ID_CONVERTTO_4BPP, OnConvertto4bpp)
	ON_UPDATE_COMMAND_UI(ID_CONVERTTO_4BPP, OnUpdateConvertto4bpp)
	ON_COMMAND(ID_CONVERTTO_8BPP, OnConvertto8bpp)
	ON_UPDATE_COMMAND_UI(ID_CONVERTTO_8BPP, OnUpdateConvertto8bpp)
	ON_COMMAND(ID_CONVERTTO_16BPP, OnConvertto16bpp)
	ON_UPDATE_COMMAND_UI(ID_CONVERTTO_16BPP, OnUpdateConvertto16bpp)
	ON_COMMAND(ID_CONVERTTO_24BPP, OnConvertto24bpp)
	ON_UPDATE_COMMAND_UI(ID_CONVERTTO_24BPP, OnUpdateConvertto24bpp)
	ON_COMMAND(ID_CONVERTTO_32BPP, OnConvertto32bpp)
	ON_UPDATE_COMMAND_UI(ID_CONVERTTO_32BPP, OnUpdateConvertto32bpp)
	ON_COMMAND(ID_FILE_INFOABOUTBITMAP, OnFileInfoaboutbitmap)
	ON_UPDATE_COMMAND_UI(ID_FILE_INFOABOUTBITMAP, OnUpdateFileInfoaboutbitmap)
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	ON_UPDATE_COMMAND_UI(ID_EDIT_COPY, OnUpdateEditCopy)
	ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
	ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, OnUpdateEditPaste)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBitmapViewerDoc construction/destruction

CBitmapViewerDoc::CBitmapViewerDoc()
{
	m_pBitmap = NULL;
	m_pBitmapPalette = NULL;
}

CBitmapViewerDoc::~CBitmapViewerDoc()
{
	ASSERT( m_pBitmap==NULL );
	ASSERT( m_pBitmapPalette==NULL );
}

/////////////////////////////////////////////////////////////////////////////
// CBitmapViewerDoc serialization

void CBitmapViewerDoc::Serialize(CArchive& ar)
{
	ASSERT( FALSE );

	if (ar.IsStoring()) {
		// TODO: add storing code here
	}
	else {
		// TODO: add loading code here
	}
}

/////////////////////////////////////////////////////////////////////////////
// CBitmapViewerDoc diagnostics

#ifdef _DEBUG
void CBitmapViewerDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CBitmapViewerDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CBitmapViewerDoc commands

CDib* CBitmapViewerDoc::GetBitmap()
{
	return m_pBitmap;
}

CPalette* CBitmapViewerDoc::GetPalette()
{
	return m_pBitmapPalette;
}

void CBitmapViewerDoc::UpdateAllDocs()
{
	CDocTemplate* pTemplate = GetDocTemplate();
	POSITION posDoc = pTemplate->GetFirstDocPosition();

	while ( posDoc!=NULL ) {
		CBitmapViewerDoc* pDoc =
			(CBitmapViewerDoc*)pTemplate->GetNextDoc(posDoc);
		ASSERT_KINDOF( CBitmapViewerDoc, pDoc );
		pDoc->UpdateAllViews(NULL);
	}
}

BOOL CBitmapViewerDoc::OnNewDocument()
{
	TRACE( "No new documents are allowed!!\n" );
	return FALSE;
}

void CBitmapViewerDoc::DeleteContents() 
{
	CDocument::DeleteContents();

	if ( m_pBitmap!=NULL ) {
		delete m_pBitmap;
		m_pBitmap = NULL;
	}

	if ( m_pBitmapPalette!=NULL ) {
		delete m_pBitmapPalette;
		m_pBitmapPalette = NULL;
	}
}

BOOL CBitmapViewerDoc::OnOpenDocument(LPCTSTR lpszPathName)
{
	DeleteContents();
	SetModifiedFlag( FALSE );

	ASSERT( m_pBitmap==NULL );

	CString sPathName = lpszPathName;
	CString sReport;

// If you are using Windows NT, try to compare display output
// performance for packed and section DIBs:

//	m_pBitmap = new CPackedDib();
	m_pBitmap = new CSectionDib();

	BOOL bResult = m_pBitmap->ReadBMP( sPathName, sReport );

// If m_pBitmap points to CDibSection, try to experiment
// with file mapped bitmaps:
/*
	BOOL bResult = ((CSectionDib*)m_pBitmap)->
		CreateAsFileMapping( sPathName, sReport );
*/

	if ( !bResult ) {
		delete m_pBitmap;
		m_pBitmap = NULL;
		AfxMessageBox( sReport );

		return FALSE;
	}

	m_pBitmapPalette = new CPalette;
	if ( !m_pBitmap->CreatePalette(m_pBitmapPalette) ) {
		delete m_pBitmapPalette;
		m_pBitmapPalette = NULL;
	}

	return bResult;
}

BOOL CBitmapViewerDoc::OnSaveDocument(LPCTSTR lpszPathName)
{
	ASSERT( m_pBitmap!=NULL );

	CString sPathName = lpszPathName;
	CString sReport;

	BOOL bResult = m_pBitmap->WriteBMP( sPathName, sReport );
	if ( !bResult )
		AfxMessageBox( sReport );

	return bResult;
}

void CBitmapViewerDoc::OnConvertto1bpp() 
{
	ASSERT_VALID( m_pBitmap );

	CWaitCursor Wait;

	CDib* pNewDib = new CPackedDib;
	CDibUt::ConvertTo1Bpp( m_pBitmap, pNewDib );

	delete m_pBitmap;
	m_pBitmap = pNewDib;

	UpdateAllViews( NULL );
}

void CBitmapViewerDoc::OnUpdateConvertto1bpp(CCmdUI* pCmdUI) 
{
	if ( m_pBitmap==NULL )
		pCmdUI->Enable(FALSE);
	else
		pCmdUI->Enable( CDibUt::CanConvertToBpp( m_pBitmap, 1 ) );
}

void CBitmapViewerDoc::OnConvertto4bpp() 
{
	ASSERT_VALID( m_pBitmap );

	CWaitCursor Wait;

	CDib* pNewDib = new CPackedDib;
	CDibUt::ConvertTo4Bpp( m_pBitmap, pNewDib );

	delete m_pBitmap;
	m_pBitmap = pNewDib;

	UpdateAllViews( NULL );
}

void CBitmapViewerDoc::OnUpdateConvertto4bpp(CCmdUI* pCmdUI) 
{
	if ( m_pBitmap==NULL )
		pCmdUI->Enable(FALSE);
	else
		pCmdUI->Enable( CDibUt::CanConvertToBpp( m_pBitmap, 4 ) );
}

void CBitmapViewerDoc::OnConvertto8bpp() 
{
	ASSERT_VALID( m_pBitmap );

	CWaitCursor Wait;

	CDib* pNewDib = new CPackedDib;
	CDibUt::ConvertTo8Bpp( m_pBitmap, pNewDib );

	delete m_pBitmap;
	m_pBitmap = pNewDib;

	UpdateAllViews( NULL );
}

void CBitmapViewerDoc::OnUpdateConvertto8bpp(CCmdUI* pCmdUI) 
{
	if ( m_pBitmap==NULL )
		pCmdUI->Enable(FALSE);
	else
		pCmdUI->Enable( CDibUt::CanConvertToBpp( m_pBitmap, 8 ) );
}

void CBitmapViewerDoc::OnConvertto16bpp()
{
	ASSERT_VALID( m_pBitmap );

	CWaitCursor Wait;

	CDib* pNewDib = new CPackedDib;
	CDibUt::ConvertTo16Bpp( m_pBitmap, pNewDib );

	delete m_pBitmap;
	m_pBitmap = pNewDib;

	UpdateAllViews( NULL );
}

void CBitmapViewerDoc::OnUpdateConvertto16bpp(CCmdUI* pCmdUI) 
{
	if ( m_pBitmap==NULL )
		pCmdUI->Enable(FALSE);
	else
		pCmdUI->Enable( CDibUt::CanConvertToBpp( m_pBitmap, 16 ) );
}

void CBitmapViewerDoc::OnConvertto24bpp() 
{
	ASSERT_VALID( m_pBitmap );

	CWaitCursor Wait;

	CDib* pNewDib = new CPackedDib;
	CDibUt::ConvertTo24Bpp( m_pBitmap, pNewDib );

	delete m_pBitmap;
	m_pBitmap = pNewDib;

	UpdateAllViews( NULL );
}

void CBitmapViewerDoc::OnUpdateConvertto24bpp(CCmdUI* pCmdUI) 
{
	if ( m_pBitmap==NULL )
		pCmdUI->Enable(FALSE);
	else
		pCmdUI->Enable( CDibUt::CanConvertToBpp( m_pBitmap, 24 ) );
}

void CBitmapViewerDoc::OnConvertto32bpp() 
{
	ASSERT_VALID( m_pBitmap );

	CWaitCursor Wait;

	CDib* pNewDib = new CPackedDib;
	CDibUt::ConvertTo32Bpp( m_pBitmap, pNewDib );

	delete m_pBitmap;
	m_pBitmap = pNewDib;

	UpdateAllViews( NULL );
}

void CBitmapViewerDoc::OnUpdateConvertto32bpp(CCmdUI* pCmdUI) 
{
	if ( m_pBitmap==NULL )
		pCmdUI->Enable(FALSE);
	else
		pCmdUI->Enable( CDibUt::CanConvertToBpp( m_pBitmap, 32 ) );
}

void CBitmapViewerDoc::OnFileInfoaboutbitmap() 
{
	CBitmapInfoDlg dlgBMInfo( m_pBitmap, GetPathName() );

	dlgBMInfo.DoModal();
}

void CBitmapViewerDoc::OnUpdateFileInfoaboutbitmap(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( m_pBitmap!=NULL );
}

void CBitmapViewerDoc::OnEditCopy() 
{
	m_pBitmap->CopyToClipboard();
}

void CBitmapViewerDoc::OnUpdateEditCopy(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( m_pBitmap!=NULL );
}

void CBitmapViewerDoc::OnEditPaste() 
{
	if ( m_pBitmap->PasteFromClipboard() ) {
		SetModifiedFlag( TRUE );
		UpdateAllViews( NULL, REFIT_AND_UPDATE );
	}
}

void CBitmapViewerDoc::OnUpdateEditPaste(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( m_pBitmap!=NULL && CDib::CanPasteFromClipboard() );
}

