////////////////////////////////////////////////////////////////////////////////
//
//  BitmapViewerView.cpp
//
//  This source file is a supplement to the article "Windows i Bitmape".
//  The article was published in the #23 and #25 issues of the "PC" computer
//  monthly, published in Belgrade, Yugoslavia by "PC Press"
//
//  You can freely use, modify and redistribute this file, provided that
//  this header is kept unchanged
//
//                                                                 Ivica Nikolic
//                                                      ivica@galeb.etf.bg.ac.yu
//
////////////////////////////////////////////////////////////////////////////////

// BitmapViewerView.cpp : implementation of the CBitmapViewerView class
//

#include "stdafx.h"
#include "BitmapViewer.h"

#include "BitmapViewerDoc.h"
#include "BitmapViewerView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CBitmapViewerView

IMPLEMENT_DYNCREATE(CBitmapViewerView, CScrollView)

BEGIN_MESSAGE_MAP(CBitmapViewerView, CScrollView)
	//{{AFX_MSG_MAP(CBitmapViewerView)
	//}}AFX_MSG_MAP
// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CScrollView::OnFilePrintPreview)
// User messages
	ON_MESSAGE( WM_REALIZE_PALETTE, OnRealizePalette )
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBitmapViewerView construction/destruction

CBitmapViewerView::CBitmapViewerView()
{
	// TODO: add construction code here

}

CBitmapViewerView::~CBitmapViewerView()
{
}

BOOL CBitmapViewerView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CScrollView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CBitmapViewerView drawing

void CBitmapViewerView::OnDraw(CDC* pDC)
{
	CBitmapViewerDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CDib* pDib = pDoc->GetBitmap();
	if ( pDib==NULL )
		return;
	if ( !pDib->IsValid() )
		return;

	pDC->SaveDC();

	CPalette* ppalBitmapPalette = pDoc->GetPalette();
	if ( ppalBitmapPalette!=NULL ) {
// Select palette as background here. If this view is in foreground, palette
// has already been selected and realized as foreground in OnRealizePalette
		pDC->SelectPalette( ppalBitmapPalette, TRUE );
		pDC->RealizePalette();
	}

	CRect rcDst;
	CRect rcSrc;
	CPoint ptSrc;

	if ( pDC->IsPrinting() ) {
		int nXSize = pDC->GetDeviceCaps( HORZRES );
		int nYSize = pDC->GetDeviceCaps( VERTRES );
		double nXAspect = (double)pDC->GetDeviceCaps( ASPECTX );
		double nYAspect = (double)pDC->GetDeviceCaps( ASPECTY );

		rcDst.left = 0;
		rcDst.top = 0;

		if ( (nXSize*nXAspect)/(nYSize*nYAspect) >
			pDib->GetWidth()/(double)pDib->GetHeight() )
		{
			rcDst.right = (int)( nYSize *
				(nYAspect*pDib->GetWidth()) / (nXAspect*pDib->GetHeight()) );
			rcDst.bottom = nYSize;
		}
		else
		{
			rcDst.right = nXSize;
			rcDst.bottom = (int)( nXSize *
				(nXAspect*pDib->GetHeight()) / (nYAspect*pDib->GetWidth()) );
		}

		rcSrc = pDib->GetExtent();
		ptSrc = CPoint( 0, 0 );
	}
	else {
		CRect rcClipBox;
		pDC->GetClipBox( &rcClipBox );

		rcDst = rcClipBox;
		rcSrc = rcClipBox;
		ptSrc = CPoint( rcClipBox.left, rcClipBox.top );
	}

// It is advisable, for performance reasons, to avoid use of StretchDIBits and
// SetDIBitsToDevice whenever possible
	if ( pDib->IsKindOf(RUNTIME_CLASS(CSectionDib)) )
	{
		CSectionDC dcSect;
		dcSect.AttachToDib( *(CSectionDib*)pDib );

		pDC->StretchBlt( rcDst.left, rcDst.top,
			rcDst.Width(), rcDst.Height(),
			&dcSect, rcSrc.left, rcSrc.top,
			rcSrc.Width(), rcSrc.Height(),
			SRCCOPY
		);
		dcSect.DetachFromDib();
	}
	else
		pDib->StretchDIBits( *pDC, rcDst, rcSrc );

	pDC->RestoreDC(-1);
}

void CBitmapViewerView::OnInitialUpdate()
{
	CScrollView::OnInitialUpdate();
	RefitToBitmap();
}

/////////////////////////////////////////////////////////////////////////////
// CBitmapViewerView printing

BOOL CBitmapViewerView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CBitmapViewerView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CBitmapViewerView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CBitmapViewerView diagnostics

#ifdef _DEBUG
void CBitmapViewerView::AssertValid() const
{
	CScrollView::AssertValid();
}

void CBitmapViewerView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}

CBitmapViewerDoc* CBitmapViewerView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CBitmapViewerDoc)));
	return (CBitmapViewerDoc*)m_pDocument;
}
#endif //_DEBUG


void CBitmapViewerView::RefitToBitmap()
{
	CDib* pDib = GetDocument()->GetBitmap();
	CSize sizeTotal;

	if ( pDib==NULL )
		sizeTotal.cx = sizeTotal.cy = 0;
	else if ( !pDib->IsValid() )
		sizeTotal.cx = sizeTotal.cy = 0;
	else
		sizeTotal = pDib->GetSize();

	SetScrollSizes(MM_TEXT, sizeTotal);
}

/////////////////////////////////////////////////////////////////////////////
// CBitmapViewerView message handlers

void CBitmapViewerView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	if ( lHint==CBitmapViewerDoc::REFIT_AND_UPDATE ) {
		RefitToBitmap();
		Invalidate();
	}
	else
		CScrollView::OnUpdate( pSender, lHint, pHint );
}

LRESULT CBitmapViewerView::OnRealizePalette( WPARAM wParam, LPARAM lParam )
{
	CBitmapViewerDoc* pDoc = GetDocument();
	CPalette *ppalDib = pDoc->GetPalette();

	if ( ppalDib!=NULL ) {

		CWnd* pMainWnd = AfxGetMainWnd();
		CClientDC dcApp( pMainWnd );
		CPalette* ppalOld = dcApp.SelectPalette( ppalDib, ((HWND)wParam)!=m_hWnd );

		if ( ppalOld!=NULL ) {
			UINT nColorsChanged = dcApp.RealizePalette();

			if ( nColorsChanged>0 )
				pDoc->UpdateAllDocs();

			dcApp.SelectPalette( ppalOld, TRUE );
		}
	}

	return 0L;
}

void CBitmapViewerView::OnActivateView(BOOL bActivate, CView* pActivateView, CView* pDeactiveView) 
{
	CScrollView::OnActivateView(bActivate, pActivateView, pDeactiveView);

	if (bActivate)
		SendMessage( WM_REALIZE_PALETTE, (WPARAM)m_hWnd, 0 );
}
