////////////////////////////////////////////////////////////////////////////////
//
//  Dib.inl
//
//  This source file is a supplement to the article "Windows i Bitmape".
//  The article was published in the #23 and #25 issues of the "PC" computer
//  monthly, published in Belgrade, Yugoslavia by "PC Press"
//
//  You can freely use, modify and redistribute this file, provided that
//  this header is kept unchanged
//
//                                                                 Ivica Nikolic
//                                                      ivica@galeb.etf.bg.ac.yu
//
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
//  Cast operators and constructors of the CPixelXX classes. These functions
//  enable easy interchange of data between DIBs in different non-indexed
//  formats:
//

inline CPixel16::CPixel16( const CPixel24& _pix24 )
{
	b = bConvTable256to32[_pix24.b];
	g = bConvTable256to32[_pix24.g];
	r = bConvTable256to32[_pix24.r];
}

inline CPixel16::CPixel16( const CPixel32& _pix32 )
{
	b = bConvTable256to32[_pix32.b];
	g = bConvTable256to32[_pix32.g];
	r = bConvTable256to32[_pix32.r];
}

inline CPixel16::operator const CPixel24()
{
	return CPixel24(
		bConvTable32to256[b],
		bConvTable32to256[g],
		bConvTable32to256[r]
	);
}

inline CPixel16::operator const CPixel32()
{
	return CPixel32(
		bConvTable32to256[b],
		bConvTable32to256[g],
		bConvTable32to256[r]
	);
}

inline CPixel24::CPixel24( const CPixel16& _pix16 )
{
	b = bConvTable32to256[_pix16.b];
	g = bConvTable32to256[_pix16.g];
	r = bConvTable32to256[_pix16.r];
}

inline CPixel24::CPixel24( const CPixel32& _pix32 )
{
	b = _pix32.b;
	g = _pix32.g;
	r = _pix32.r;
}

inline CPixel24::operator const CPixel16()
{
	return CPixel16(
		bConvTable256to32[b],
		bConvTable256to32[g],
		bConvTable256to32[r]
	);
}

inline CPixel24::operator const CPixel32()
{
	return CPixel32(
		b,
		g,
		r
	);
}


inline CPixel32::CPixel32( const CPixel16& _pix16 )
{
	b = bConvTable32to256[_pix16.b];
	g = bConvTable32to256[_pix16.g];
	r = bConvTable32to256[_pix16.r];
}

inline CPixel32::CPixel32( const CPixel24& _pix24 )
{
	b = _pix24.b;
	g = _pix24.g;
	r = _pix24.r;
}

inline CPixel32::operator const CPixel16()
{
	return CPixel16(
		bConvTable256to32[b],
		bConvTable256to32[g],
		bConvTable256to32[r]
	);
}

inline CPixel32::operator const CPixel24()
{
	return CPixel24(
		b,
		g,
		r
	);
}


////////////////////////////////////////////////////////////////////////////////
//  Non-debug, inline versions of the data-access CDib member functions.
//  Theese functions are intended for well-encapsulated yet still
//  high-performance access to individual pixels.
//
//  See debug versions (implemented in Dib.cpp) for comments
//

#if !defined(_DEBUG)

inline BYTE CDib::GetPix1(int x, int y)
{
	BYTE bSevenMinusRemainder = BYTE(x & 7) ^ BYTE(7);
	BYTE bMask = BYTE(1) << bSevenMinusRemainder;
	return (m_pbRowCache[y][x>>3] & bMask) >> bSevenMinusRemainder;
}

inline void CDib::SetPix1(int x, int y, BYTE Val)
{
	BYTE bMask = BYTE(1) << (BYTE(x&7) ^ BYTE(7));
	if ( (Val&1) != 0 )
		m_pbRowCache[y][x>>3] |= bMask;
	else
		m_pbRowCache[y][x>>3] &= ~bMask;
}

inline BYTE CDib::GetPix4(int x, int y)
{
	if ( (x&1) == 0 )
		return m_pbRowCache[y][x>>1] >> 4;
	else
		return m_pbRowCache[y][x>>1] & (BYTE)0x0F;
}

inline void CDib::SetPix4(int x, int y, BYTE Val)
{
	BYTE* pByte = m_pbRowCache[y] + (x>>1);
	if ( (x&1) == 0 )
		*pByte = (*pByte & (BYTE)0x0F) | (Val<<4);
	else
		*pByte = (*pByte & (BYTE)0xF0) | (Val&0x0F);
}

inline BYTE CDib::GetPix8(int x, int y)
{
	return m_pbRowCache[y][x];
}

inline void CDib::SetPix8(int x, int y, BYTE Val)
{
	m_pbRowCache[y][x] = Val;
}

inline const CPixel16 CDib::GetPix16(int x, int y)
{
	return * (CPixel16*) &( ((WORD*)(m_pbRowCache[y])) [x] );
}

inline void CDib::SetPix16(int x, int y, const CPixel16 Val)
{
	((WORD*)(m_pbRowCache[y])) [x] = LOWORD(*(LONG*)&Val);
}

inline const CPixel24 CDib::GetPix24(int x, int y)
{
	return ((CPixel24*)(m_pbRowCache[y])) [x];
}

inline void CDib::SetPix24(int x, int y, const CPixel24 Val)
{
	((CPixel24*)(m_pbRowCache[y])) [x] = Val;
}

inline const CPixel32 CDib::GetPix32(int x, int y)
{
	return ((CPixel32*)(m_pbRowCache[y])) [x];
}

inline void CDib::SetPix32(int x, int y, const CPixel32 Val)
{
	((CPixel32*)(m_pbRowCache[y])) [x] = Val;
}

#endif // _DEBUG
