////////////////////////////////////////////////////////////////////////////////
//
//  DibUt.cpp
//
//  This source file is a supplement to the article "Windows i Bitmape".
//  The article was published in the #23 and #25 issues of the "PC" computer
//  monthly, published in Belgrade, Yugoslavia by "PC Press"
//
//  You can freely use, modify and redistribute this file, provided that
//  this header is kept unchanged
//
//                                                                 Ivica Nikolic
//                                                      ivica@galeb.etf.bg.ac.yu
//
////////////////////////////////////////////////////////////////////////////////



//
//  This source file contains implementations of class utility functions
//  declared in DibUt.h. All functions are fairly self explanatory. Furthermore,
//  they are almost trivial - they serve only as an example of how to use CDib's
//  data access functions.
//

#include "stdafx.h"

#include "DibUt.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

////////////////////////////////////////////////////////////////////////////////
// Small helper used internaly
static void CopyDibColorTable( CDib* pdibSrc, CDib* pdibDst )
{
	ASSERT_VALID( pdibSrc );
	ASSERT_VALID( pdibDst );
	ASSERT( pdibSrc->GetColorTableSize()==pdibDst->GetColorTableSize() );

	CArray<RGBQUAD,RGBQUAD&> Colors;
	pdibSrc->GetDIBColorTable( 0, pdibSrc->GetColorTableSize(), Colors );
	pdibDst->SetDIBColorTable( 0, pdibDst->GetColorTableSize(), Colors );
}

BOOL CDibUt::CanConvertToBpp( CDib* pdibSrc, int nBpp )
{
	ASSERT_VALID( pdibSrc );
	ASSERT( nBpp==1 || nBpp==4 || nBpp==8 || nBpp==16 || nBpp==24 || nBpp==32 );

	switch ( pdibSrc->GetBitsPerPixel() ) {
	case 1:
		return nBpp==4 || nBpp==8;
	case 4:
		return nBpp==8;
	case 8:
		return FALSE;
	case 16:
		return nBpp==24 || nBpp==32;
	case 24:
		return nBpp==16 || nBpp==32;
	case 32:
		return nBpp==16 || nBpp==24;
	default:
		ASSERT( FALSE );
		return FALSE;
	}
}

void CDibUt::ConvertTo32Bpp( CDib* pdibSrc, CDib* pdibDst )
{
	ASSERT_VALID( pdibSrc );
	ASSERT( pdibDst!=NULL );
	ASSERT( pdibSrc->GetBitsPerPixel()==16 || pdibSrc->GetBitsPerPixel()==24 );

	CWaitCursor Wait;

	int
		w = pdibSrc->GetWidth(),
		h = pdibSrc->GetHeight(),
		x, y;

	pdibDst->CreateBlank( w, h, 32, pdibSrc->GetColorTableSize() );
	CopyDibColorTable( pdibSrc, pdibDst );

	for ( y=h-1; y>=0; y-- )
		for ( x=w-1; x>=0; x-- )
			pdibDst->SetPix32( x, y, pdibSrc->GetPix32Clever(x,y) );
}

void CDibUt::ConvertTo24Bpp( CDib* pdibSrc, CDib* pdibDst )
{
	ASSERT_VALID( pdibSrc );
	ASSERT( pdibDst!=NULL );
	ASSERT( pdibSrc->GetBitsPerPixel()==16 || pdibSrc->GetBitsPerPixel()==32 );

	CWaitCursor Wait;

	int
		w = pdibSrc->GetWidth(),
		h = pdibSrc->GetHeight(),
		x, y;

	pdibDst->CreateBlank( w, h, 24, pdibSrc->GetColorTableSize() );
	CopyDibColorTable( pdibSrc, pdibDst );

	for ( y=h-1; y>=0; y-- )
		for ( x=w-1; x>=0; x-- )
			pdibDst->SetPix24( x, y, (const CPixel24)pdibSrc->GetPix32Clever(x,y) );
}

void CDibUt::ConvertTo16Bpp( CDib* pdibSrc, CDib* pdibDst )
{
	ASSERT_VALID( pdibSrc );
	ASSERT( pdibDst!=NULL );
	ASSERT( pdibSrc->GetBitsPerPixel()==24 || pdibSrc->GetBitsPerPixel()==32 );

	CWaitCursor Wait;

	int
		w = pdibSrc->GetWidth(),
		h = pdibSrc->GetHeight(),
		x, y;

	pdibDst->CreateBlank( w, h, 16, pdibSrc->GetColorTableSize() );
	CopyDibColorTable( pdibSrc, pdibDst );

	for ( y=h-1; y>=0; y-- )
		for ( x=w-1; x>=0; x-- )
			pdibDst->SetPix16( x, y, (const CPixel16)pdibSrc->GetPix32Clever(x,y) );
}

void CDibUt::ConvertTo8Bpp( CDib* pdibSrc, CDib* pdibDst )
{
	ASSERT_VALID( pdibSrc );
	ASSERT( pdibDst!=NULL );
	ASSERT( pdibSrc->GetBitsPerPixel()==1 || pdibSrc->GetBitsPerPixel()==4 );

	CWaitCursor Wait;

	int
		w = pdibSrc->GetWidth(),
		h = pdibSrc->GetHeight(),
		x, y;

	pdibDst->CreateBlank( w, h, 8, pdibSrc->GetColorTableSize() );
	CopyDibColorTable( pdibSrc, pdibDst );

	for ( y=h-1; y>=0; y-- )
		for ( x=w-1; x>=0; x-- )
			pdibDst->SetPix8( x, y, pdibSrc->GetPix8Clever(x,y) );
}

void CDibUt::ConvertTo4Bpp( CDib* pdibSrc, CDib* pdibDst )
{
	ASSERT_VALID( pdibSrc );
	ASSERT( pdibDst!=NULL );
	ASSERT( pdibSrc->GetBitsPerPixel()==1 );

	CWaitCursor Wait;

	int
		w = pdibSrc->GetWidth(),
		h = pdibSrc->GetHeight(),
		x, y;

	pdibDst->CreateBlank( w, h, 4, pdibSrc->GetColorTableSize() );
	CopyDibColorTable( pdibSrc, pdibDst );

	for ( y=h-1; y>=0; y-- )
		for ( x=w-1; x>=0; x-- )
			pdibDst->SetPix4( x, y, pdibSrc->GetPix8Clever(x,y) );
}

void CDibUt::ConvertTo1Bpp( CDib* pdibSrc, CDib* pdibDst )
{
	ASSERT( FALSE );
}

void CDibUt::SplitRGBChannels( CDib* pdibSrc,
	CDib* pdibDstB, CDib* pdibDstG, CDib* pdibDstR )
{
	ASSERT_VALID( pdibSrc );
	ASSERT(
		pdibSrc->GetBitsPerPixel()==16 ||
		pdibSrc->GetBitsPerPixel()==24 ||
		pdibSrc->GetBitsPerPixel()==32
	);

	int nWidth = pdibSrc->GetWidth();
	int nHeight = pdibSrc->GetHeight();

	pdibDstB->CreateBlank( nWidth, nHeight, 8, 256 );
	pdibDstG->CreateBlank( nWidth, nHeight, 8, 256 );
	pdibDstR->CreateBlank( nWidth, nHeight, 8, 256 );

	int x, y;
	CPixel32 pix32;

	for ( y=nHeight-1; y>=0; y-- )
		for ( x=nWidth-1; x>=0; x-- ) {
			pix32 = pdibSrc->GetPix32Clever( x, y );
			pdibDstB->SetPix8( x, y, pix32.b );
			pdibDstG->SetPix8( x, y, pix32.g );
			pdibDstR->SetPix8( x, y, pix32.r );
		}

	CArray<RGBQUAD,RGBQUAD&> rgbColors;
	rgbColors.SetSize( 256 );
	int i;

	for ( i=0; i<256; i++ ) {
		rgbColors[i].rgbBlue = i;
		rgbColors[i].rgbGreen = 0;
		rgbColors[i].rgbRed = 0;
		rgbColors[i].rgbReserved = 0;
	}
	pdibDstB->SetDIBColorTable( 0, 256, rgbColors );

	for ( i=0; i<256; i++ ) {
		rgbColors[i].rgbBlue = 0;
		rgbColors[i].rgbGreen = i;
		rgbColors[i].rgbRed = 0;
		rgbColors[i].rgbReserved = 0;
	}
	pdibDstG->SetDIBColorTable( 0, 256, rgbColors );

	for ( i=0; i<256; i++ ) {
		rgbColors[i].rgbBlue = 0;
		rgbColors[i].rgbGreen = 0;
		rgbColors[i].rgbRed = i;
		rgbColors[i].rgbReserved = 0;
	}
	pdibDstR->SetDIBColorTable( 0, 256, rgbColors );
}

void CDibUt::CombineRGBChannels( CDib* pdibSrcB, CDib* pdibSrcG,
	CDib* pdibSrcR, CDib* pdibDst, int nDstBitsPerPixel )
{
	ASSERT_VALID( pdibSrcB );
	ASSERT_VALID( pdibSrcG );
	ASSERT_VALID( pdibSrcR );

	ASSERT( pdibSrcB->GetBitsPerPixel()==8 );
	ASSERT( pdibSrcG->GetBitsPerPixel()==8 );
	ASSERT( pdibSrcR->GetBitsPerPixel()==8 );

	ASSERT( pdibSrcB->GetSize()==pdibSrcG->GetSize() );
	ASSERT( pdibSrcB->GetSize()==pdibSrcR->GetSize() );

	ASSERT(
		nDstBitsPerPixel==16 ||
		nDstBitsPerPixel==24 ||
		nDstBitsPerPixel==32
	);


	int nWidth = pdibSrcB->GetWidth();
	int nHeight = pdibSrcB->GetHeight();

	pdibDst->CreateBlank( nWidth, nHeight, nDstBitsPerPixel, 0 );

	int x, y;
	CPixel32 pix32;

	for ( y=nHeight-1; y>=0; y-- )
		for ( x=nWidth-1; x>=0; x-- ) {
			pix32.b = pdibSrcB->GetPix8( x, y );
			pix32.g = pdibSrcG->GetPix8( x, y );
			pix32.r = pdibSrcR->GetPix8( x, y );

			pdibDst->SetPix32Clever( x, y, pix32 );
		}
}
